 /*///////////////////////////////////////////////////////////////
// ArduLift Control Program
//
//    by Gary Liming
//
//    Version 0.9 Proof of Concept       January 2017
//    Version 1.0 Debugged and Frozen
//
//    January 2017
//
////////////////////////////////////////////////////////////////*/
#include "Ardulift.h"
#include <Wire.h> 
#include <LiquidCrystal_I2C.h>

LiquidCrystal_I2C disp(0x3F,20,4);  // set the LCD address to 0x3F and the I2C pins

#define DEBUG 1

#define StatReady       1  // status definitions
#define StatMovingUp    2
#define StatMovingDown  3
#define StatError       4
#define StatUpLimit     5  // we are at Up limit
#define StatDownLimit   6  // we are at Down Limit
#define OK              1  // some general indicators used for readability
#define ERR            -1
#define TRUE            1
#define FALSE           0
#define ON              1
#define OFF             0
#define UP              1
#define DOWN            0
#define METRIC          2  // symbol for units
#define IMPERIAL        1  // symbol for units
#define FAST            1
#define SLOW            0  // symbol for speed setting

char Version[4] = "1.0";               // Version of this software
unsigned CurStat = 0;                  // global status indicator
unsigned LastStat = 0;                 // place to remember the last one
unsigned CurSpeed = 0;                 // current speed setting
unsigned CurSpeedDelay = 0L;           // resulting speed dealy in microseconds
unsigned LastSpeed = 12;               // place to remember the last setting
float CurInchPos = 0.0;                // current number of inches above zero
float CurMMPos = 0.0;                  // current number of mm above zero
long CurStepPos = 0L;                  // current number of steps above or below zero
long LastStepPos = 0xFFFFFF;
float LiftStepsPerInch = 0.0;          // calculated steps per inch
int WhichUnits = IMPERIAL;             // start with Imperial, but the switch position will set it
int testflag = FALSE;                  // general flag
int UnitsFlag = FALSE;                 // flag to indicate the units have changed
int UpLimitFlag = FALSE;               // True if we are at up limit position
int DownLimitFlag = FALSE;             // True if we are at down limit position
int ProbeStat = FALSE;                 // flag to indicate Probe sequence in progress
int ProbeTouch = FALSE;                // flag to indicate probe touching the bit

void setup()  {
  #ifdef DEBUG
    Serial.begin (57600);
    delay(100);
    Serial.println("");
    Serial.println(F("ArduLift Debug Start"));
  #endif
  disp.init();          // init the data structure
  disp.backlight();     // turn on backlight 
  
  pinMode(UnitsSwitchPin, INPUT_PULLUP);        // init the input switches, and enable pullup resistors on them
  pinMode(ZeroSwitchPin, INPUT_PULLUP);
  pinMode(UpSwitchPin, INPUT_PULLUP);     
  pinMode(DownSwitchPin, INPUT_PULLUP);    
  pinMode(LimitUpSwitchPin, INPUT_PULLUP); 
  pinMode(LimitDownSwitchPin, INPUT_PULLUP); 
  pinMode(StartProbePin, INPUT_PULLUP);
  pinMode(ProbeSwitchPin, INPUT_PULLUP);
  pinMode(Lift_DIR_Pin, OUTPUT);
  pinMode(Lift_STEP_Pin, OUTPUT);

  digitalWrite(Lift_DIR_Pin, UP);              // ensure motor is off for now
  digitalWrite(Lift_STEP_Pin, OFF);
  
  LiftStepsPerInch = (float)(LiftStepsPerRevolution * LiftMicroSteps) * (float)LiftThreadsPerInch;
  ShowSplash();                        // announce the splash screen
  CurStat = StatReady;                 // set default state to ready;
  TestLimits();                        // check to see if we are at either limit postion
  GetSpeed();                          // check position of speed switch
  TestUnits();                         // check units switch
  InitScreen();                        // draw screen lables
  #ifdef DEBUG
    Serial.println(F("Setup Complete..."));   // Report how the setup went
    Serial.print(F("LiftStepsPerInch = "));
    Serial.print(LiftStepsPerInch);
    Serial.print(F(", Units are "));
    if (WhichUnits == IMPERIAL)
      Serial.println(F("Imperial"));
    if (WhichUnits == METRIC)
      Serial.println(F("Metric"));
    Serial.print(F("Speed is set to "));  
    Serial.println(CurSpeed);
  #endif
}

void loop()  {                         // main loop is just to wait for user input
  
  GetSpeed();                          // check position of speed switch
  TestUnits();                         // check units switch
  TestLimits();                        // check to see if we are at either limit postion
  TestZero();                          // check to see if zero button is pushed
  if (TestProbeStart())                // check to see if probe sequence start is pushed
    ProbeSequence();                   // if so, do the probe thing
  if (TestMove())                      // Are we moving yet?
    MoveLift();                        // if so, move it out
  ShowStatus();                        // update the screen
}

void ShowSplash(void)  {               // display the init screen for a few seconds
  disp.clear();
  disp.setCursor(2,0);
  disp.print(F("ArduLift Control"));
  disp.setCursor(4,2);
  disp.print(F("Version "));
  disp.setCursor(12,2);
  disp.print(Version);
  delay(SplashTime);
  disp.clear();
}

void InitScreen(void)  {                    // paint the labels on the screen
  disp.clear();
  disp.setCursor(2,0);                      // display program title
  disp.print(F("ArduLift Control"));
  disp.setCursor(2,1);                      // display status
  disp.print(F("Status:")); 
  disp.setCursor(3,2);                      // display Speed setting
  disp.print(F("Speed:"));
  disp.setCursor(0,3);                      // display Position
  disp.print(F("Position:")); 
}

void ShowStatus(void)  {                    // display status

  disp.setCursor(10,1);
  if (CurStat != LastStat)  {               // erase any previous status, if it's changed
    disp.print(F("         "));           
    disp.setCursor(10,1);
  }  
  switch (CurStat) {
    case StatReady:
      disp.print(F("Ready"));
      break;
    case StatError:
       disp.print(F("Error"));
      break;
    case StatMovingUp:
      disp.print(F("Raise"));
      break;
    case StatMovingDown:
      disp.print(F("Lower"));
      break;
    case StatDownLimit:
      disp.print(F("Down Limit"));
      break;
    case StatUpLimit:
      disp.print(F("Up Limit"));
      break;
    default:
        disp.print(F("Unknown"));    
  }
  LastStat = CurStat;

  if (CurSpeed != LastSpeed)  {
    disp.setCursor(10,2);                     // display Speed setting
    disp.print("  ");                         // erase any previous speed 
    disp.setCursor(10,2);
    disp.print(CurSpeed);
    LastSpeed = CurSpeed;
  }
  ShowPos();
  ShowProbeHit();                                                 // check for and display probe touch
  return;
}
void ShowPos(void)  {
  if ((LastStepPos != CurStepPos) || (UnitsFlag)) {               // if we've changed position or units, display it
    disp.setCursor(10,3);                                         // erase all digits of current position
    disp.print(F("       "));
    disp.setCursor(10,3);
    if (WhichUnits == IMPERIAL) {                                 // display SAE
      CurInchPos = (float)CurStepPos/(float)LiftStepsPerInch;
      disp.print(CurInchPos,3);
      disp.setCursor(17,3);
      disp.print(F("in"));
    }
    if (WhichUnits == METRIC) {                                   // display Metric
      CurMMPos = ((float)CurStepPos/(float)LiftStepsPerInch)*25.40L;
      disp.print(CurMMPos,2);
      disp.setCursor(17,3); 
      disp.print(F("mm"));
    }
    LastStepPos = CurStepPos;
    UnitsFlag = 0;
  }
  return;
}

void ShowProbeHit(void) {
  disp.setCursor(19,0);
  if (TestProbe())                                        // display probe contact, if any
    disp.print(F("*"));
  else
    disp.print(" ");                                      // otherwise, clear it
  return;
}

int GetSpeed(void)  {
  unsigned oldSpeed = CurSpeed;
  
  CurSpeed = analogRead(SpeedKnobPin);

  CurSpeed = map(CurSpeed, 1023, 0, 1, 10);              // make the range of values 1 to 10
  if (oldSpeed != CurSpeed) {
    CurSpeedDelay = map(CurSpeed,1,10,1000,170);         // high speed delay for this motor is 170
    #ifdef DEBUG                                         // low speed delay is about 1000 
      Serial.print(F("Analog Speed changed to "));
      Serial.println(CurSpeed);
    #endif
    return(TRUE); 
  } else
    return(FALSE);
}
int TestMove(void) {
  if (digitalRead(UpSwitchPin) == OFF) {                 // read and test speed switch
    delay(DebounceTime);                                 // if so, wait a bit to settle
    if (digitalRead(UpSwitchPin) == OFF)  {              // if its still low, its been pressed.
      if (UpLimitFlag == FALSE) {
        CurStat = StatMovingUp;  
        #ifdef DEBUG
          Serial.println(F("Up Button Pressed"));
        #endif
        return(TRUE);
      } 
    }                        
  }
  if (digitalRead(DownSwitchPin) == OFF) {       // read and test speed switch
    delay(DebounceTime);                           // if so, wait a bit to settle
    if (digitalRead(DownSwitchPin) == OFF)  {      // if its still low, its been pressed.
      if (DownLimitFlag == FALSE) {
        CurStat = StatMovingDown;  
        #ifdef DEBUG
          Serial.println(F("Down Button Pressed"));
        #endif
        return(TRUE);
      }
    }                        
  }
  return(FALSE);
}

int TestUnits(void)  {
  int oldUnits = WhichUnits;

  if (digitalRead(UnitsSwitchPin) == ON) {       // read and test speed switch
    delay(DebounceTime);                           // if so, wait a bit to settle
    if (digitalRead(UnitsSwitchPin) == ON)       // if its still low, its been pressed.
      WhichUnits = IMPERIAL;                       // update the speed   
  } else {
    if (digitalRead(UnitsSwitchPin) == OFF) {   
      delay(DebounceTime);
      if (digitalRead(UnitsSwitchPin) == OFF)      // if its still low, its been pressed.
        WhichUnits = METRIC;                       // update the units 
    }
  }   
  if (oldUnits != WhichUnits)  {
    UnitsFlag = 1;
    ShowStatus();
    #ifdef DEBUG
       Serial.print(F("Units Changed to "));
       if (WhichUnits == IMPERIAL)
         Serial.println(F("Imperial"));
       else
         Serial.println(F("Metric"));
    #endif
    return(TRUE);      
  } else
    return(FALSE); 
}
int TestZero(void)  {

  if (CurStepPos == 0)                         // if already zero, we're done
    return(FALSE);
  if (digitalRead(ZeroSwitchPin) == OFF) {        // read and test zero switch
    delay(DebounceTime);                          // if so, wait a bit to settle
    if (digitalRead(ZeroSwitchPin) == OFF) {      // if its still low, its been pressed.
      CurStepPos = 0L;                            // update the steps   
      CalcPos();                                  // update the units position  
      #ifdef DEBUG
        Serial.println(F("Zero Button Pressed"));
      #endif
      return(TRUE);
    }  
  }
  return(FALSE); 
}

int TestProbeStart(void)  {                       // check to see if user pressed the probe start switch
  ProbeStat = FALSE;
  if (digitalRead(StartProbePin) == OFF) {        // read and test zero switch
    delay(DebounceTime);                          // if so, wait a bit to settle
    if (digitalRead(StartProbePin) == OFF) {      // if its still low, its been pressed.
      ProbeStat = TRUE;
      #ifdef DEBUG
        Serial.println(F("Probe Button Pressed"));
      #endif  
    }  
  }
  return(ProbeStat); 
}

int TestProbe(void)  {                            // check to see if the bit hit the probe
  ProbeTouch = FALSE;
  if (digitalRead(ProbeSwitchPin) == OFF) {        // read and test zero switch
    delay(DebounceTime);                          // if so, wait a bit to settle
    if (digitalRead(ProbeSwitchPin) == OFF){       // if its still low, its been pressed.
      ProbeTouch = TRUE; 
      #ifdef DEBUG
        Serial.println(F("Probe contact made"));
      #endif 
    }  
  }
  return(ProbeTouch); 
}

int TestLimits(void)  {
  if (TestUpLimit())
    return(TRUE);
  if (TestDownLimit())
    return(TRUE);
  return(FALSE);    
}
int TestUpLimit(void)  {
  int oldStat = CurStat;
  
  CurStat = StatReady;                             // default to ready
  if (digitalRead(LimitUpSwitchPin) == ON) {       // at upper switch?
    delay(DebounceTime);                           // if so, wait a bit to settle
    if (digitalRead(LimitUpSwitchPin) == ON) {     // if its still there, it's been pressed
      CurStat = StatUpLimit;                       // update new status
      if (oldStat != CurStat){
        #ifdef DEBUG
          Serial.println(F("Upper limit switch hit"));
        #endif 
        UpLimitFlag = TRUE;
      }  
    }     
  }
  return(UpLimitFlag);
}
int TestDownLimit(void) { 
  int oldStat = CurStat;
   
  CurStat = StatReady;
  if (digitalRead(LimitDownSwitchPin) == ON) {      // at lower switch?
    delay(DebounceTime);                            // if so, wait a bit to settle
    if (digitalRead(LimitDownSwitchPin) == ON) {    // if its still low, its been pressed.
      CurStat = StatDownLimit;                      // update new status   
      if (oldStat != CurStat)   {
        #ifdef DEBUG
          Serial.println(F("Lower limit switch hit"));
        #endif
        DownLimitFlag = TRUE;
      }  
    }
  }
  return(DownLimitFlag);                                // otherwise, nothing to report
}  

void CalcPos(void) {
  if (CurStepPos == 0L)  {
    CurInchPos = 0.0;            // current number of inches above zero
    CurMMPos = 0.0;  
  } else 
    CurInchPos = CurStepPos/LiftStepsPerInch;
  return;
}

void ProbeSequence (void) {     // do the probe - raise the lift and watch for probe closure
  
  digitalWrite(Lift_DIR_Pin, UP);                 // set direction to UP
  for (;;) {                    
    if(digitalRead(LimitUpSwitchPin) == ON) {     //if we topped out, stop
      UpLimitFlag = TRUE;
      ProbeStat = FALSE;
      CurStat = StatReady;
      return;
    }
    if (digitalRead(ProbeSwitchPin) == OFF) {
      CurStepPos = 0.0;                           // ok, we are at zero, set the new position
      CalcPos();
      CurStat = StatReady;
      ProbeStat = FALSE;
      return;
    }
    CurStepPos++;
    digitalWrite(Lift_STEP_Pin, ON);
    delayMicroseconds(CurSpeedDelay);
    digitalWrite(Lift_STEP_Pin, OFF);
    delayMicroseconds(CurSpeedDelay);
//    bump motor
    if (CurStepPos%(CurSpeed*20) == 0)
      ShowPos();                       
  }
  return;
}  

void MoveLift(void) {
  disp.setCursor(10,1);                       // update screen with status once
  disp.print(F("          "));            
  if (CurStat == StatMovingUp) {              // if we're going up
    disp.setCursor(10,1);
    disp.print(F("Raise"));                   // bump upsteps 
    digitalWrite(Lift_DIR_Pin, UP);                 // set direction to UP
    while (digitalRead(UpSwitchPin) == OFF) { // up button is pressed
      if (digitalRead(LimitUpSwitchPin) == ON) {  // and we are not at limit position
        UpLimitFlag = TRUE;
        CurStat = StatReady;
        break;
      }
      ++CurStepPos;
      digitalWrite(Lift_STEP_Pin, ON);
      delayMicroseconds(CurSpeedDelay);
      digitalWrite(Lift_STEP_Pin, OFF);
      delayMicroseconds(CurSpeedDelay);
      if (CurStepPos%(CurSpeed*50) == 0)
        ShowPos();
    }
    DownLimitFlag = FALSE; 
  } else if (CurStat == StatMovingDown){
    disp.setCursor(10,1);
    disp.print(F("Lower"));
    UpLimitFlag = FALSE;
    digitalWrite(Lift_DIR_Pin, DOWN);                // set direction to DOWN 
    while (digitalRead(DownSwitchPin) == OFF) {      // loop while button is down and not at limit
      if (digitalRead(LimitDownSwitchPin) == ON)  {  // if we are at limit, we're done
        DownLimitFlag = TRUE;
        CurStat = StatReady;
        break;
      }
      --CurStepPos;                                  // bump the position
      digitalWrite(Lift_STEP_Pin, ON);               // pulse the motor
      delayMicroseconds(CurSpeedDelay);
      digitalWrite(Lift_STEP_Pin, OFF);
      delayMicroseconds(CurSpeedDelay);
      if (CurStepPos%(CurSpeed*50) == 0)             // update the screen occasionally
        ShowPos();
    }
  }
  disp.setCursor(10,1);
  disp.print(F("           "));
  CurStat = StatReady;
  return;
}


